# -*- coding: utf-8 -*-


class Risk:
    LICENSE_RISK = "License Risk"
    OPERATIONAL_RISK = "Operational Risk"
    SECURITY_RISK = "Security Risk"


class Level:
    CRITICAL = "Critical"
    HIGH = "High"
    MEDIUM = "Medium"
    LOW = "Low"
    NONE = "None"


class Component:
    COMPONENT = "Component"
    SOURCE = "Source"
    MATCH_TYPE = "Match Type"
    USAGE = "Usage"
    LICENSE = "License"
    SECURITY_RISK = "Security Risk"
    OPERATIONAL_RISK = "Operational Risk"


class File:
    ID = "ID"
    NAME = "Name"
    LINES = "Lines"
    OSS_LINES = "OSS Lines"
    MATCHED = "Matched"
    PURL = "PKG URL"
    VENDOR = "Vendor"
    COMPONENT = "Component"
    VERSION = "Version"
    LATEST = "Latest"
    URL = "URL"
    RELEASE_DATE = "Release Date"
    FILE = "File"
    DEPENDENCIES = "Dependencies"
    LICENSES = "Licenses"
    VULNERABILITIES = "Vulnerabilities"



# -*- coding: utf-8 -*-

import openpyxl
import os

from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import Paragraph, SimpleDocTemplate, Table

from ossreport.proto.proto import Component, File, Level, Risk


risk_head = {
    "A": Risk.__name__.capitalize(),
    "B": Level.CRITICAL,
    "C": Level.HIGH,
    "D": Level.MEDIUM,
    "E": Level.LOW,
    "F": Level.NONE,
}

component_head = {
    "A": Component.COMPONENT,
    "B": Component.SOURCE,
    "C": Component.MATCH_TYPE,
    "D": Component.USAGE,
    "E": Component.LICENSE,
    "F": Component.SECURITY_RISK,
    "G": Component.OPERATIONAL_RISK,
}

file_head = {
    "A": File.ID,
    "B": File.NAME,
    "C": File.LINES,
    "D": File.OSS_LINES,
    "E": File.MATCHED,
    "F": File.PURL,
    "G": File.VENDOR,
    "H": File.COMPONENT,
    "I": File.VERSION,
    "J": File.LATEST,
    "K": File.URL,
    "L": File.RELEASE_DATE,
    "M": File.FILE,
    "N": File.DEPENDENCIES,
    "O": File.LICENSES,
    "P": File.VULNERABILITIES,
}


class PrinterException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Printer(object):
    def __init__(self, risks, components, files, name):
        self._risks = risks
        self._components = components
        self._files = files
        self._name = name

    def _pdf(self):
        def _styling_title(style):
            return style["Title"]

        def _styling_head(style):
            return style["Heading3"]

        def _styling_table():
            return [
                ("ALIGN", (0, 0), (-1, -1), "CENTER"),
                ("FONTSIZE", (0, 0), (-1, 0), 8),
                ("FONTSIZE", (0, 1), (-1, -1), 6),
                ("GRID", (0, 0), (-1, -1), 0.1, colors.black),
                ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ]

        def _write_table(head, data, style):
            content = [[head[key] for key in sorted(head.keys())]]
            for _, val in data.items():
                for v in val:
                    buf = [v[head[_k]] for _k in sorted(head.keys())]
                    content.append(buf)
            return Table(data=content, style=style, colWidths=["*"])

        stylesheet = getSampleStyleSheet()
        story = [
            Paragraph("SecTrend SCA Report", _styling_title(stylesheet)),
            Paragraph("", _styling_head(stylesheet)),
            _write_table(risk_head, self._risks, _styling_table()),
            Paragraph("", _styling_head(stylesheet)),
            _write_table(component_head, self._components, _styling_table()),
        ]
        doc = SimpleDocTemplate(self._name)
        doc.build(story)

    def _xlsx(self):
        def _styling_head(sheet, head):
            for item in head.keys():
                sheet[item + "1"].alignment = openpyxl.styles.Alignment(
                    horizontal="center", shrink_to_fit=True, vertical="center"
                )
                sheet[item + "1"].font = openpyxl.styles.Font(bold=True, name="Calibri")
            sheet.freeze_panes = sheet["A2"]

        def _styling_content(sheet, head, rows):
            for key in head.keys():
                for row in range(rows):
                    sheet[key + str(row + 2)].alignment = openpyxl.styles.Alignment(
                        horizontal="center", vertical="center"
                    )
                    sheet[key + str(row + 2)].font = openpyxl.styles.Font(
                        bold=False, name="Calibri"
                    )

        def _write_table(book, head, data):
            sheet = book.create_sheet()
            sheet.append([head[key] for key in sorted(head.keys())])
            head_len = 0
            for key, val in data.items():
                sheet.title = key
                for v in val:
                    buf = [v[head[_k]] for _k in sorted(head.keys())]
                    head_len = len(buf)
                    sheet.append(buf)
            _styling_head(sheet, head)
            _styling_content(sheet, head, head_len)

        wb = openpyxl.Workbook()
        wb.remove(wb.active)
        _write_table(wb, risk_head, self._risks)
        _write_table(wb, component_head, self._components)
        _write_table(wb, file_head, self._files)
        wb.save(filename=self._name)

    def run(self):
        func = Printer.__dict__.get(
            os.path.splitext(self._name)[1].replace(".", "_"), None
        )
        if func is not None:
            func(self)



# -*- coding: utf-8 -*-

import sys

from ossreport.analyzer.analyzer import Analyzer, AnalyzerException
from ossreport.builder.builder import Builder, BuilderException
from ossreport.cmd.argument import Argument
from ossreport.cmd.banner import BANNER
from ossreport.config.config import Config, ConfigException
from ossreport.logger.logger import Logger
from ossreport.printer.printer import Printer, PrinterException


def main():
    print(BANNER)

    argument = Argument()
    arg = argument.parse(sys.argv)

    try:
        config = Config()
        config.config_file = arg.config_file
        config.license_file = arg.license_file
        config.output_file = arg.output_file
        config.scanoss_file = arg.scanoss_file
    except ConfigException as e:
        Logger.error(str(e))
        return -1

    Logger.info("report running")

    try:
        analyzer = Analyzer(config.config_file, config.license_file)
    except AnalyzerException as e:
        Logger.error(str(e))
        return -2

    try:
        builder = Builder(config.scanoss_file, analyzer)
        risks, components, files = builder.run()
    except BuilderException as e:
        Logger.error(str(e))
        return -3

    try:
        printer = Printer(risks, components, files, config.output_file)
        printer.run()
    except PrinterException as e:
        Logger.error(str(e))
        return -4

    Logger.info("report exiting")

    return 0



__author__ = "Jia Jia"
__author_email__ = "angersax@sina.com"
__description__ = "SCANOSS Report"
__license__ = "Apache-2.0"
__title__ = "ossreport"
__url__ = "https://github.com/craftslab/ossreport"
__version__ = "1.1.0"



# -*- coding: utf-8 -*-

import argparse
import os

from ossreport.__version__ import __version__


class Argument(object):
    def __init__(self):
        self._parser = argparse.ArgumentParser(description="SCANOSS Report")
        self._add()

    def _add(self):
        self._parser.add_argument(
            "--config-file",
            action="store",
            default=os.path.join(os.getcwd(), "config", "config.yml"),
            dest="config_file",
            help="config file (default: %s)"
            % os.path.join("ossreport", "config", "config.yml"),
            required=False,
        )
        self._parser.add_argument(
            "--license-file",
            action="store",
            default=os.path.join(os.getcwd(), "data", "license.json"),
            dest="license_file",
            help="license file (default: %s)"
            % os.path.join("ossreport", "data", "license.json"),
            required=False,
        )
        self._parser.add_argument(
            "--output-file",
            action="store",
            dest="output_file",
            help="output file (.pdf|.xlsx)",
            required=True,
        )
        self._parser.add_argument(
            "--scanoss-file",
            action="store",
            dest="scanoss_file",
            help="scanoss file (.json)",
            required=True,
        )
        self._parser.add_argument(
            "-v", "--version", action="version", version=__version__
        )

    def parse(self, argv):
        return self._parser.parse_args(argv[1:])



# -*- coding: utf-8 -*-

from ossreport.proto.proto import Level


class AnalyzerException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Analyzer(object):
    def __init__(self, config, license):
        pass

    def license_risk(self, data):
        # TODO
        return Level.NONE

    def operational_risk(self, data):
        # TODO
        return Level.NONE



# -*- coding: utf-8 -*-

from ossreport.proto.proto import Risk, Level, Component, File


class BuilderException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Builder(object):
    _COMPONENT_SEP = ","
    _VULNERABILITIES_SEP = "/"

    _SOURCE_MATCHES = "Matches"

    def __init__(self, scanoss, analyzer):
        self._scanoss = scanoss
        self._analyzer = analyzer

    def _build_risks(self, data):
        def _build_license_risk(data):
            critical, high, medium, low, none = 0, 0, 0, 0, 0
            for component in data:
                for license in component[Component.LICENSE].split(self._COMPONENT_SEP):
                    level = license.split(" ")[0]
                    if level == Level.CRITICAL[0].upper():
                        critical += 1
                    elif level == Level.HIGH[0].upper():
                        high += 1
                    elif level == Level.MEDIUM[0].upper():
                        medium += 1
                    elif level == Level.LOW[0].upper():
                        low += 1
                    elif level == Level.NONE[0].upper():
                        none += 1
            return {
                Risk.__name__.capitalize(): Risk.LICENSE_RISK,
                Level.CRITICAL: critical,
                Level.HIGH: high,
                Level.MEDIUM: medium,
                Level.LOW: low,
                Level.NONE: none,
            }

        def _build_operational_risk(data):
            critical, high, medium, low, none = 0, 0, 0, 0, 0
            # TODO
            return {
                Risk.__name__.capitalize(): Risk.OPERATIONAL_RISK,
                Level.CRITICAL: critical,
                Level.HIGH: high,
                Level.MEDIUM: medium,
                Level.LOW: low,
                Level.NONE: none,
            }

        def _build_security_risk(data):
            critical, high, medium, low, none = 0, 0, 0, 0, 0
            for item in data:
                c, h, m, l, n = item[Component.SECURITY_RISK].split(" ")
                if int(c.lstrip(Level.CRITICAL[0].upper())) > 0:
                    critical += 1
                if int(h.lstrip(Level.HIGH[0].upper())) > 0:
                    high += 1
                if int(m.lstrip(Level.MEDIUM[0].upper())) > 0:
                    medium += 1
                if int(l.lstrip(Level.LOW[0].upper())) > 0:
                    low += 1
                if int(n.lstrip(Level.NONE[0].upper())) > 0:
                    none += 1
            return {
                Risk.__name__.capitalize(): Risk.SECURITY_RISK,
                Level.CRITICAL: critical,
                Level.HIGH: high,
                Level.MEDIUM: medium,
                Level.LOW: low,
                Level.NONE: none,
            }

        return {
            Risk.__name__: [
                _build_license_risk(data),
                _build_operational_risk(data),
                _build_security_risk(data),
            ]
        }

    def _build_components(self, data):
        def _fetch(data):
            critical, high, medium, low, none = 0, 0, 0, 0, 0
            for item in data[File.VULNERABILITIES].split(" "):
                severity = item.split(self._VULNERABILITIES_SEP)[-1].upper()
                if severity == Level.CRITICAL.upper():
                    critical += 1
                elif severity == Level.HIGH.upper():
                    high += 1
                elif severity == Level.MEDIUM.upper():
                    medium += 1
                elif severity == Level.LOW.upper():
                    low += 1
                else:
                    none += 1
            return {
                Component.COMPONENT: " ".join(
                    [data[File.COMPONENT].capitalize(), data[File.VERSION]]
                ),
                Component.SOURCE: " ".join(["1", self._SOURCE_MATCHES]),
                Component.MATCH_TYPE: data[File.ID].capitalize(),
                Component.USAGE: "",
                Component.LICENSE: " ".join(
                    [
                        self._analyzer.license_risk(data[File.LICENSES])[0].upper(),
                        data[File.LICENSES],
                    ]
                ),
                Component.SECURITY_RISK: " ".join(
                    [
                        Level.CRITICAL[0].upper() + str(critical),
                        Level.HIGH[0].upper() + str(high),
                        Level.MEDIUM[0].upper() + str(medium),
                        Level.LOW[0].upper() + str(low),
                        Level.NONE[0].upper() + str(none),
                    ]
                ),
                Component.OPERATIONAL_RISK: "",
            }

        def _update(data, new):
            data[Component.SOURCE] = " ".join(
                [
                    str(
                        int(data[Component.SOURCE].split(" ")[0])
                        + int(new[Component.SOURCE].split(" ")[0])
                    ),
                    self._SOURCE_MATCHES,
                ]
            )
            data[Component.MATCH_TYPE] = (
                data[Component.MATCH_TYPE]
                if new[Component.MATCH_TYPE] in data[Component.MATCH_TYPE]
                else self._COMPONENT_SEP.join(
                    [data[Component.MATCH_TYPE], new[Component.MATCH_TYPE]]
                )
            )
            data[Component.USAGE] = (
                data[Component.USAGE]
                if new[Component.USAGE] in data[Component.USAGE]
                else self._COMPONENT_SEP.join(
                    [data[Component.USAGE], new[Component.USAGE]]
                )
            )
            data[Component.LICENSE] = (
                data[Component.LICENSE]
                if new[Component.LICENSE] in data[Component.LICENSE]
                else self._COMPONENT_SEP.join(
                    [data[Component.LICENSE], new[Component.LICENSE]]
                )
            )
            c, h, m, l, n = new[Component.SECURITY_RISK].split(" ")
            critical, high, medium, low, none = data[Component.SECURITY_RISK].split(" ")
            critical = Level.CRITICAL[0].upper() + str(
                int(critical.lstrip(Level.CRITICAL[0].upper()))
                + int(c.lstrip(Level.CRITICAL[0].upper()))
            )
            high = Level.HIGH[0].upper() + str(
                int(high.lstrip(Level.HIGH[0].upper()))
                + int(h.lstrip(Level.HIGH[0].upper()))
            )
            medium = Level.MEDIUM[0].upper() + str(
                int(medium.lstrip(Level.MEDIUM[0].upper()))
                + int(m.lstrip(Level.MEDIUM[0].upper()))
            )
            low = Level.LOW[0].upper() + str(
                int(low.lstrip(Level.LOW[0].upper()))
                + int(l.lstrip(Level.LOW[0].upper()))
            )
            none = Level.NONE[0].upper() + str(
                int(none.lstrip(Level.NONE[0].upper()))
                + int(n.lstrip(Level.NONE[0].upper()))
            )
            data[Component.SECURITY_RISK] = " ".join(
                [critical, high, medium, low, none]
            )
            data[Component.OPERATIONAL_RISK] = (
                data[Component.OPERATIONAL_RISK]
                if new[Component.OPERATIONAL_RISK] in data[Component.OPERATIONAL_RISK]
                else self._COMPONENT_SEP.join(
                    [data[Component.OPERATIONAL_RISK], new[Component.OPERATIONAL_RISK]]
                )
            )
            return data

        buf = {}
        for item in data:
            b = _fetch(item)
            if b[Component.COMPONENT] not in buf.keys():
                buf[b[Component.COMPONENT]] = b
            else:
                buf[b[Component.COMPONENT]] = _update(buf[b[Component.COMPONENT]], b)
        return {Component.__name__: [v for v in buf.values()]}

    def _build_files(self):
        buf = {File.__name__: []}
        for key, val in self._scanoss.items():
            for v in val:
                if v["id"] == "none":
                    continue
                buf[File.__name__].append(
                    {
                        File.ID: v["id"],
                        File.NAME: key,
                        File.LINES: v["lines"],
                        File.OSS_LINES: v["oss_lines"],
                        File.MATCHED: v["matched"],
                        File.PURL: " ".join([p for p in v["purl"]]),
                        File.VENDOR: v["vendor"],
                        File.COMPONENT: v["component"],
                        File.VERSION: v["version"],
                        File.LATEST: v["latest"],
                        File.URL: v["url"],
                        File.RELEASE_DATE: v["release_date"],
                        File.FILE: v["file"],
                        File.DEPENDENCIES: " ".join([d for d in v["dependencies"]]),
                        File.LICENSES: " ".join([lic["name"] for lic in v["licenses"]]),
                        File.VULNERABILITIES: " ".join(
                            [
                                vul["CVE"] + self._VULNERABILITIES_SEP + vul["severity"]
                                for vul in v["vulnerabilities"]
                            ]
                        ),
                    }
                )
        return buf

    def run(self):
        files = self._build_files()
        components = self._build_components(files[File.__name__])
        risks = self._build_risks(components[Component.__name__])
        return risks, components, files



# -*- coding: utf-8 -*-

import json
import os
import yaml


class ConfigFile:
    APIVERSION = "apiVersion"
    KIND = "kind"
    METADATA = "metadata"
    SPEC = "spec"


class MetaData:
    NAME = "name"


class ConfigException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Config(object):
    def __init__(self):
        self._config_file = None
        self._license_file = None
        self._output_file = None
        self._scanoss_file = None

    @property
    def config_file(self):
        return self._config_file

    @config_file.setter
    def config_file(self, name):
        if not isinstance(name, str) or len(name.strip()) == 0:
            raise ConfigException("config invalid")
        name = name.strip()
        if not name.endswith(".yml") and not name.endswith(".yaml"):
            raise ConfigException("suffix invalid")
        if not os.path.exists(name):
            raise ConfigException("%s not found" % name)
        with open(name) as file:
            self._config_file = yaml.load(file, Loader=yaml.FullLoader)
        if self._config_file is None:
            raise ConfigException("config invalid")

    @property
    def license_file(self):
        return self._license_file

    @license_file.setter
    def license_file(self, name):
        if not isinstance(name, str) or len(name.strip()) == 0:
            raise ConfigException("config invalid")
        name = name.strip()
        if not name.endswith(".json"):
            raise ConfigException("suffix invalid")
        if not os.path.exists(name):
            raise ConfigException("%s not found" % name)
        with open(name, encoding="utf-8") as file:
            self._license_file = json.load(file)
        if self._license_file is None:
            raise ConfigException("config invalid")

    @property
    def output_file(self):
        return self._output_file

    @output_file.setter
    def output_file(self, name):
        if not isinstance(name, str) or len(name.strip()) == 0:
            raise ConfigException("output invalid")
        name = name.strip()
        if not name.endswith(".pdf") and not name.endswith(".xlsx"):
            raise ConfigException("suffix invalid")
        if os.path.exists(name):
            raise ConfigException("%s already exist" % name)
        self._output_file = name

    @property
    def scanoss_file(self):
        return self._scanoss_file

    @scanoss_file.setter
    def scanoss_file(self, name):
        if not isinstance(name, str) or len(name.strip()) == 0:
            raise ConfigException("config invalid")
        name = name.strip()
        if not name.endswith(".json"):
            raise ConfigException("suffix invalid")
        if not os.path.exists(name):
            raise ConfigException("%s not found" % name)
        with open(name, encoding="utf-8") as file:
            self._scanoss_file = json.load(file)
        if self._scanoss_file is None:
            raise ConfigException("config invalid")
